@@ -385,6 +385,40 @@ DataProcessor::CreateReadInfo(string read_info_file,
 	return true;
 }
 
+/*
+ *--------------------------------------------------------------------------------------
+ *        Class:  DataProcessor
+ *       Method:  WriteReadInfo
+ *  Description:  Write read information to a file
+ *        Param:  read_info_file  [i] : See -read_info for the format of this file. 
+ *                read_infos      [o] : The pointer in this array will be newed. They
+ *                                      should be deleted outside.
+ *       Return:  
+ *--------------------------------------------------------------------------------------
+ */
+bool
+DataProcessor::WriteReadInfo(string read_info_file, 
+						  vector<ReadInfoBase*>& read_infos,
+						  vector<string>& mapping_files)
+{
+	fstream outfile;
+	outfile.open(read_info_file.data(), ios::out);
+	if (!outfile.is_open())
+	{
+		cerr << "File " << read_info_file.data() << " can not be opened" << endl;
+		return false;
+	}
+
+	for (unsigned i = 0; i < read_infos.size(); ++i)
+	{
+		outfile << mapping_files[i] << " ";
+		read_infos[i]->Write(&outfile);
+		outfile << endl;
+	}
+	
+	outfile.close();
+	return true;
+}
 /*
  *--------------------------------------------------------------------------------------
  *        Class:  DataProcessor
@@ -621,6 +655,8 @@ DataProcessor::LoadShortReads(string from_file, map<string, vector<Exon> >& exon
 	cout << line_cnt << " lines have been scaned." << "   "  
 		 << read_cnt << " reads have been loaded successfully." << endl;
 
+	p_read_info->mTotalReadCnt = read_cnt;
+
 	infile.close();
 
 	return true;
@@ -1729,19 +1765,24 @@ DataProcessor::ExtractJunctionRef(map_str2vec_exon& exons, map_str2vec_int& exon
 	// The junction refseq generated in this way is not exactly right
 	// When the length of the exon is less than the length of the reads
 	// the junction refseq may be impossible to be constructed.
+	bool b_out_of_range = false;
 	while (iter_first != first_seg.end())
 	{
 		if (pos2refseq.find(*iter_chr) != pos2refseq.end())
 		{
 			map_int_str& a_map = pos2refseq[*iter_chr];
-			if (a_map[*iter_first].size() < cross_len)
+			if (a_map[*iter_first].size() < cross_len && !b_out_of_range)
 			{
-				cerr << "ERROR : on chromosome " << *iter_chr << ", position " << *iter_first << " is out of range" << endl;
+				cerr << "ERROR : on chromosome " << *iter_chr << ", boundary " << *iter_first << " is out of range. "
+					 << "Similar errors will be suppressed" << endl;
+				b_out_of_range = true;
 				continue;
 			}
-			if (a_map[*iter_second].size() < cross_len)
+			if (a_map[*iter_second].size() < cross_len && !b_out_of_range)
 			{
-				cerr << "ERROR : on chromosome " << *iter_chr << ", position " << *iter_second << " is out of range" << endl;
+				cerr << "ERROR : on chromosome " << *iter_chr << ", boundary " << *iter_second << " is out of range. "
+					 << "Similar errors will be suppressed" << endl;
+				b_out_of_range = true;
 				continue;
 			}
 
