@@ -77,8 +77,6 @@ class MishMash : public ToolBase
 			cout <<                                                      endl;
 			cout << "          -w_exp   : Write the expression levels of instances to a file. -ins is needed." << endl;
 			cout <<                                                      endl;
-			cout << "          -gen_hui : Generate random instances needed Hui's matlab code" << endl;
-			cout <<                                                      endl;
 			cout << "          -gen_tss_pas : Extract boundaries, gene range, TSSs and PASs from instances" << endl;
 			cout <<                                                      endl;
 			cout << "          -comp_iso <T/F, pred_iso_file, ben_iso_file> : Compare the prediced isoforms with " << endl;
@@ -125,7 +123,6 @@ class MishMash : public ToolBase
 			bool b_help = true;
 			bool b_ins_stat = false;
 			bool b_comp_exp = false;
-			bool b_gen_hui = false;
 			bool b_gen_tss_pas = false;
 			bool b_gen_exon = false;
 			bool b_gen_iso = false;
@@ -155,8 +152,6 @@ class MishMash : public ToolBase
 					output_file = argv[++i];
 				else if (strcmp(argv[i], "-ins_stat") == 0)
 					b_ins_stat = true;
-				else if (strcmp(argv[i], "-gen_hui") == 0)
-					b_gen_hui = true;
 				else if (strcmp(argv[i], "-gen_tss_pas") == 0)
 					b_gen_tss_pas = true;
 				else if (strcmp(argv[i], "-gen_exon") == 0)
@@ -206,7 +201,7 @@ class MishMash : public ToolBase
 				}
 			}
 
-			if (!b_ins_stat && !b_comp_exp && !b_gen_hui && !b_w_exp && !b_rel_diff && 
+			if (!b_ins_stat && !b_comp_exp && !b_w_exp && !b_rel_diff && 
 				!b_gen_tss_pas && !b_gen_exon && !b_gen_iso && !b_comp_iso && !b_spliced_iso && !b_comp_interval)
 			{
 				ArgMissing("Job is missing");
@@ -253,18 +248,6 @@ class MishMash : public ToolBase
 			{
 				CompareInterval(first_file, second_file);
 			}
-			else if (b_gen_hui)
-			{
-				InstanceWriter writer(mpOutput);
-				writer.SetConfidencelevel(0.05);
-				writer.SetWriteFormat(InstanceWriter::WRITE_HUI);
-
-				InstanceReader reader(ins_file);
-				reader.SetHandler(&writer);
-				reader.Initialize();
-				reader.Generate();
-				reader.CleanUp();
-			}
 			else if (b_gen_tss_pas)
 			{
 				InstanceWriter writer(mpOutput);
