@@ -113,6 +113,7 @@ class ReleaseInterface : public ToolBase
 			string read_info_file = "";
 			string output_file = "";
 			bool b_strand_specific = false;
+			bool b_update_read_info = false;
 			string ins_file = "";
 			int start_pos_of_first_nt = 0;
 			string bound_file = "";
@@ -130,7 +131,7 @@ class ReleaseInterface : public ToolBase
 			int min_iso = 8;
 			int iso_scale = 9;
 			bool b_use_provided_exons = false;
-			double noise_level = 0;
+			double noise_level = 1;
 			int cal_exp_method= 2;
 			bool b_bs2 = true;
 			string pred_file = "";
@@ -184,6 +185,8 @@ class ReleaseInterface : public ToolBase
 					refseq_file = argv[++i];
 				else if (strcmp(argv[i], "-read_info") == 0)
 					read_info_file = argv[++i];
+				else if (strcmp(argv[i], "-update_read_info") == 0)
+					b_update_read_info = (argv[++i][0] == 'T');
 				else if (strcmp(argv[i], "-o") == 0)
 					output_file = argv[++i];
 				else if (strcmp(argv[i], "-s") == 0)
@@ -208,8 +211,6 @@ class ReleaseInterface : public ToolBase
 					min_dup = atoi(argv[++i]);
 				else if (strcmp(argv[i], "-ps") == 0)
 					part_size = atoi(argv[++i]);
-				else if (strcmp(argv[i], "-noise") == 0)
-					noise_level = atof(argv[++i]);
 				else if (strcmp(argv[i], "-conf_level") == 0)
 					conf_level = atof(argv[++i]);
 
@@ -282,7 +283,13 @@ class ReleaseInterface : public ToolBase
 			vector<ReadInfoBase*> read_infos;
 			vector<string> mapping_files;
 			if (read_info_file != "")
+			{
 				dp.CreateReadInfo(read_info_file, read_infos, mapping_files);
+				noise_level = read_infos[0]->mNoiseLevel;
+				for (unsigned i = 1; i < read_infos.size(); ++i)
+					if (noise_level < read_infos[0]->mNoiseLevel)
+						noise_level = read_infos[0]->mNoiseLevel;
+			}
 			dp.mbStrandSpecific = b_strand_specific;
 			dp.mpOutput = p_output;
 
@@ -371,6 +378,9 @@ class ReleaseInterface : public ToolBase
 				for (unsigned i = 0; i < read_infos.size(); ++i)
 					dp.LoadShortReads(mapping_files[i], exons, read_infos[i], all_instances);
 
+				if (b_update_read_info)
+					dp.WriteReadInfo(read_info_file, read_infos, mapping_files);
+
 				InstanceWriter writer(p_output);
 				writer.Initialize();
 				for (unsigned i = 0; i < all_instances.size(); i++)
@@ -492,6 +502,9 @@ class ReleaseInterface : public ToolBase
 				for (unsigned i = 0; i < read_infos.size(); ++i)
 					dp.LoadShortReads(mapping_files[i], exons, read_infos[i], all_instances);
 
+				if (b_update_read_info)
+					dp.WriteReadInfo(read_info_file, read_infos, mapping_files);
+
 				InstanceWriter writer(p_output);
 				writer.Initialize();
 				for (unsigned i = 0; i < all_instances.size(); i++)
