@@ -20,8 +20,7 @@
 #include <string>
 #include <iostream>
 #include "InstanceHandler.h"
-#include "Utility2.h"
-#include "MishMash.h"
+#include "Utility.hpp"
 
 using namespace std;
 
@@ -35,7 +34,7 @@ class ResultSummary : public InstanceHandler
 {
 	public:
 		/* ====================  LIFECYCLE     ======================================= */
-		ResultSummary (IsoInferPE* p_infer, ostream* p_output = NULL) : InstanceHandler(p_output)
+		ResultSummary (IsoInfer* p_infer, ostream* p_output = NULL) : InstanceHandler(p_output)
 		{
 			mpInfer = p_infer;
 			mOutputFormat = 0;
@@ -58,7 +57,205 @@ class ResultSummary : public InstanceHandler
 		Initialize()
 		{
 			mPredictCnt = 0;	
-		};
+		}
+
+		// Compare two set of isoforms. predict_iso_file should be the output of OutputPrediction.
+		// benchmark_iso_file should be the output of InstanceWriter::WRITE_KNOWN 
+		void
+		PredVSBenchmark(string predict_iso_file, string benchmark_iso_file)
+		{
+			// Load benchmark file
+			fstream infile;
+			infile.open(benchmark_iso_file.data(), ios::in);
+			if (!infile.is_open())
+			{
+				cerr << "File " << benchmark_iso_file.data() << " can not be opened" << endl;
+				exit(0);
+			}
+
+			vector<vector<Gene> > known_isoforms;
+			vector<vector<double> > known_isoforms_exp;
+			vector<vector<bool> > is_high;
+
+			// read the data and fill the vector
+			vector<string> splitted;
+			int line_cnt = 0;
+			string line;
+			while (getline(infile, line))
+			{
+				line_cnt++;
+				splitted = Utility::Split(';', line);
+
+				vector<string> sub_splitted = Utility::Split('\t', splitted[0]);
+				int ins_cnt = atoi(sub_splitted[0].data());
+				if (known_isoforms.size() < ins_cnt + 1)
+				{
+					known_isoforms.resize(ins_cnt + 1);
+					known_isoforms_exp.resize(ins_cnt + 1);
+					is_high.resize(ins_cnt + 1);
+				}
+				known_isoforms[ins_cnt].resize(known_isoforms[ins_cnt].size() + 1);
+				known_isoforms[ins_cnt][known_isoforms[ins_cnt].size()-1].Read(splitted[1]);
+
+				known_isoforms_exp[ins_cnt].push_back(atof(sub_splitted[4].data()));
+				is_high[ins_cnt].push_back(sub_splitted[5]=="T");
+			}
+			cout << line_cnt << " known isoforms have been loaded" << endl;
+
+			infile.close();
+
+			// Load prediction file
+			infile.open(predict_iso_file.data(), ios::in);
+			if (!infile.is_open())
+			{
+				cerr << "File " << predict_iso_file.data() << " can not be opened" << endl;
+				exit(0);
+			}
+
+			vector<vector<Gene> > predicted_isoforms;
+
+			// read the data and fill the vector
+			line_cnt = 0;
+			while (getline(infile, line))
+			{
+				line_cnt++;
+				splitted = Utility::Split('\t', line);
+
+				int ins_cnt = atoi(splitted[0].data());
+
+				if (predicted_isoforms.size() < ins_cnt + 1)
+					predicted_isoforms.resize(ins_cnt + 1);
+				predicted_isoforms[ins_cnt].resize(predicted_isoforms[ins_cnt].size() + 1);
+
+				Gene& a_gene = predicted_isoforms[ins_cnt][predicted_isoforms[ins_cnt].size()-1];
+				int pos = line.find("\t");
+				string left_str = line.substr(pos+1, line.size()-pos-1);
+				a_gene.Read(left_str);
+			}
+
+			cout << line_cnt << " predicted isoforms have been loaded" << endl;
+
+			infile.close();
+
+			predicted_isoforms.resize(known_isoforms.size());
+
+			// Compare two set of isoforms
+			for (unsigned i = 0; i < predicted_isoforms.size(); ++i)
+			{
+				vector<int> solution;
+				for (unsigned j = 0; j < predicted_isoforms[i].size(); ++j)
+					solution.push_back(j);
+
+				bool b_output = false;
+				if (predicted_isoforms.size() - 1 == i) b_output = true;
+				
+				/*
+				cout << "On instance " << i << endl;
+				for (unsigned j = 0; j < known_isoforms[i].size(); ++j)
+					known_isoforms[i][j].Write();
+				for (unsigned j = 0; j < predicted_isoforms[i].size(); ++j)
+					predicted_isoforms[i][j].Write();
+				cout << endl;
+			    */
+
+				ResultStatistics(i, known_isoforms[i], predicted_isoforms[i], 
+						known_isoforms_exp[i], is_high[i], solution, b_output);
+			}
+		}
+
+		/*
+		 *--------------------------------------------------------------------------------------
+		 *       Class:  MishMash
+		 *      Method:  RelDiff
+		 * Description:  Compare the estimated value to the true value and calculate the relative difference. 
+		 *               The first column of each file contains the names and the second column of each file
+		 *               contains the scores. The value in the first file will be scaled by 'true scale'.
+		 *       Param:  
+		 *      Return:  
+		 *--------------------------------------------------------------------------------------
+		 */
+		bool
+		RelDiff(string true_file, string esti_file, double true_scale)
+		{
+			fstream infile;
+			infile.open(true_file.data(), ios::in);
+			if (!infile.is_open())
+			{
+				cerr << "File " << true_file.data() << " can not be opened" << endl;
+				return false;
+			}
+
+			vector<string> splitted;
+
+			map<string, double> name_value;
+
+			int line_cnt = 0;
+			string line;
+			while (getline(infile, line))
+			{
+				splitted = Utility::Split('\t', line);
+				name_value[splitted[0]] = atof(splitted[1].data()) * true_scale;
+				line_cnt++;
+			}
+
+			infile.close();
+
+			infile.open(esti_file.data(), ios::in);
+			if (!infile.is_open())
+			{
+				cerr << "File " << esti_file.data() << " can not be opened" << endl;
+				return false;
+			}
+
+			int no_match_cnt = 0;
+			int valid_cnt = 0;
+
+			double ave_square_error_sum = 0;
+
+			int range_cnt = 21;
+			vector<double> range_ratio;
+			range_ratio.assign(range_cnt, 0);
+			line_cnt = 0;
+			while (getline(infile, line))
+			{
+				splitted = Utility::Split('\t', line);
+				string name = splitted[0];
+				double value = atof(splitted[1].data());
+				if (value == 0) value = 0.000001;
+
+				if (name_value.find(name) == name_value.end())
+				{
+					no_match_cnt++;
+					continue;
+				}
+
+				valid_cnt++;
+
+				double rel_diff = abs(value - name_value[name]) / name_value[name] + abs(value - name_value[name]) / value;
+				rel_diff /= 2;
+				for (int i = 0; i < range_cnt; i++)
+				{
+					if (rel_diff <= i * 0.05) range_ratio[i] += 1;
+				}
+
+				ave_square_error_sum += (value - name_value[name]) * (value - name_value[name]);
+				// cout << value << "\t" << name_value[name] << "\t" << rel_diff << endl;
+				line_cnt++;
+			}
+
+			ave_square_error_sum /= line_cnt;
+
+			infile.close();
+
+			(*mpOutput) << "Valid : No Match = " << valid_cnt << " : " << no_match_cnt << endl;
+			for (int i = 0; i < range_cnt; i++)
+				(*mpOutput) << setw(10) << i*0.05;
+			(*mpOutput) << endl;
+			for (int i = 0; i < range_cnt; i++)
+				(*mpOutput) << setw(10) << range_ratio[i] / valid_cnt;
+			(*mpOutput) << endl;
+			(*mpOutput) << "Average square error sum = " << ave_square_error_sum << endl;
+		}		/* -----  end of method RelDiff----- */
 
 
 		virtual
@@ -87,11 +284,18 @@ class ResultSummary : public InstanceHandler
 					gene.CalculateRange();
 				}
 
-				ResultStatistics(an_instance.mExons, an_instance.mGenes, valid_genes, an_instance.mIsoExp,
-						         an_instance.mSpliceReadCnt, solution);
+				vector<vector<double> > junc_cnt;
+				an_instance.mShortReadGroup.JuncCnt(an_instance.mSegLen.size(), junc_cnt);
+
+				vector<bool> is_supported_by_junc;
+				is_supported_by_junc.assign(an_instance.mGenes.size(), false);
+				for (unsigned i = 0; i < is_supported_by_junc.size(); ++i)
+					if (IsSupportedByJunc(an_instance.mExons, an_instance.mGenes[i], junc_cnt))
+						is_supported_by_junc[i] = true;
+				ResultStatistics(an_instance.mInstanceCnt, an_instance.mGenes, valid_genes, an_instance.mIsoExp, is_supported_by_junc, solution);
 			}
 			if (1 == mOutputFormat || 2 == mOutputFormat)
-				OutputPrediction(an_instance.mExons, valid_isoforms, solution);
+				OutputPrediction(an_instance, valid_isoforms, solution);
 
 		}/* -----  end of method OnInstance  ----- */
 
@@ -113,12 +317,13 @@ class ResultSummary : public InstanceHandler
 		//  Parameters:  solution  :  The indexes of the final recovered isoforms in valid_isoforms
 		//--------------------------------------------------------------------------------------
 			void
-		ResultStatistics(const vector<Exon>& exons,
+		ResultStatistics(int ins_cnt,
 						 const vector<Gene>& known_isoforms,
 						 const vector<Gene>& valid_isoforms,
 						 const vector<double>& known_isoforms_exp, 
-						 const vector<vector<double> >& junc_cnt,
-						 const vector<int>& solution)
+						 const vector<bool>& is_supported_by_junc,
+						 const vector<int>& solution,
+						 bool b_output = true)
 		{
 			int scale_low = -15;
 			int scale_high = 10;
@@ -214,8 +419,7 @@ class ResultSummary : public InstanceHandler
 
 			int high_curr = 0;
 			for (unsigned i = 0; i < known_isoforms.size(); i++)
-				if (IsSupportedByJunc(exons, known_isoforms[i], junc_cnt))
-					high_curr++;
+				if (is_supported_by_junc[i]) high_curr++;
 
 			mHighKnown_by_IsoCnt[iso_cnt] += high_curr;
 
@@ -252,8 +456,10 @@ class ResultSummary : public InstanceHandler
 			for (unsigned i = 0; i < mHighKnown_by_IsoCnt.size(); i++)
 				high_all += mHighKnown_by_IsoCnt[i];
 
+			if (!b_output) return;
+
 			// Output result
-			(*mpOutput) << "Inst : " << mpInstance->mInstanceCnt << endl;
+			(*mpOutput) << "Inst : " << ins_cnt << endl;
 			(*mpOutput) << "Results current: " << endl;
 			(*mpOutput) << "    known_curr             : " << setw(6) << known_curr << endl;
 			(*mpOutput) << "    high_curr              : " << setw(6) << high_curr << endl;
@@ -393,11 +599,19 @@ class ResultSummary : public InstanceHandler
 				int64 end_pos = gene_exons[i].mEnd;
 				int64 start_pos = gene_exons[i+1].mStart;
 
-				int start_idx = Utility2Temp<int>::BinarySearch(start_position, start_pos);
-				if (start_position[start_idx] != start_pos) return false;
+				int start_idx = UtilityTemp<int>::BinarySearch(start_position, start_pos);
+				if (start_position[start_idx] != start_pos)
+				{
+					cerr << "Warning : the known isoform is inconsistent with the expressed segments in this instance" << endl;
+					return false;
+				}
 
-				int end_idx = Utility2Temp<int>::BinarySearch(end_position, end_pos);
-				if (end_position[end_idx] != end_pos) return false;
+				int end_idx = UtilityTemp<int>::BinarySearch(end_position, end_pos);
+				if (end_position[end_idx] != end_pos)
+				{
+					cerr << "Warning : the known isoform is inconsistent with the expressed segments in this instance" << endl;
+					return false;
+				}
 
 				//cout << end_idx << "," << start_idx << "," << junc_cnt[end_idx][start_idx] << endl;
 				if (junc_cnt[end_idx][start_idx] <= 0) return false;
@@ -406,6 +620,53 @@ class ResultSummary : public InstanceHandler
 			return true;
 		}
 
+		/*
+		 *--------------------------------------------------------------------------------------
+		 *       Class:  ResultSummary
+		 *      Method:  CompareTwoIsoforms
+		 * Description:  
+		 *       Param:  
+		 *      Return:  
+		 *--------------------------------------------------------------------------------------
+		 */
+		double
+		CompareTwoIsoforms(const Gene& first_transcript, const Gene& second_transcript)
+		{
+			double overlap_cnt = 0;
+			for (unsigned i = 0; i < first_transcript.mExons.size(); i++)
+			{
+				int first_start = first_transcript.mExons[i].mStart;
+				int first_end = first_transcript.mExons[i].mEnd;
+
+				for (unsigned j = 0; j < second_transcript.mExons.size(); j++)
+				{
+					int second_start = second_transcript.mExons[j].mStart;
+					int second_end = second_transcript.mExons[j].mEnd;
+
+					int max_start = first_start;
+					if (max_start < second_start) max_start = second_start;
+
+					int min_end = first_end;
+					if (min_end > second_end) min_end = second_end;
+
+					if (min_end - max_start > 0)
+						overlap_cnt += min_end - max_start;
+				}
+			}
+
+			int first_len = 0;
+			for (unsigned i = 0; i < first_transcript.mExons.size(); i++)
+				first_len += first_transcript.mExons[i].mEnd - first_transcript.mExons[i].mStart;
+
+			int second_len = 0;
+			for (unsigned i = 0; i < second_transcript.mExons.size(); i++)
+				second_len += second_transcript.mExons[i].mEnd - second_transcript.mExons[i].mStart;
+
+			return 2 * overlap_cnt / (first_len + second_len);
+		}		/* -----  end of method CompareTwoIsoforms  ----- */
+
+
+
 		//--------------------------------------------------------------------------------------
 		//       Class:  ResultSummary
 		//      Method:  MatchedIndex 
@@ -421,7 +682,7 @@ class ResultSummary : public InstanceHandler
 			{
 				if (gene.mStart >= genes[i].mEnd || gene.mEnd <= genes[i].mStart) continue;
 
-				double match_score = MishMash::CompareTwoIsoforms(gene, genes[i]);
+				double match_score = CompareTwoIsoforms(gene, genes[i]);
 
 				if (1 == match_score)
 					matched_idx.push_back(i);
@@ -435,13 +696,14 @@ class ResultSummary : public InstanceHandler
 		//  Parameters:  solution  :  The indexes of the final recovered isoforms in valid_isoforms
 		//--------------------------------------------------------------------------------------
 			void
-		OutputPrediction(const vector<Exon>& exons, 
+		OutputPrediction(const Instance& an_instance, 
 						 const vector<vector<bool> >& valid_isoforms, 
 						 const vector<int>& solution)
 		{
+			const vector<Exon>& exons = an_instance.mExons;
 			for (unsigned i = 0; i < solution.size(); i++)
 			{
-				(*mpOutput) << "Pred" << mPredictCnt++ << "\t" << exons[0].mChr << "\t" << (exons[0].mStrand ? '+' : '-') << "\t";
+				(*mpOutput) << an_instance.mInstanceCnt << "\tPred" << mPredictCnt++ << "\t" << exons[0].mChr << "\t" << (exons[0].mStrand ? '+' : '-') << "\t";
 				
 				const vector<bool>& an_iso = valid_isoforms[solution[i]];
 
@@ -483,7 +745,7 @@ class ResultSummary : public InstanceHandler
 		}
 
 	private:
-		IsoInferPE* mpInfer;
+		IsoInfer* mpInfer;
 
 		int mMinIsoCnt;          // Only genes with at least mMinIsoCnt isoforms will be considered
 
